#!"\xampp\perl\bin\perl.exe"
# start_server_as_admin.pl (backend start)
# (c) napp-it 2023,2024

# scriptname: start_server_as_admin.pl

# this startup script start_server_as_admin.pl as child process. It should run to BSD, Illumos/Solaris, Linux, OSX and Windows member servers
# It starts the backend server for remote management from a Windows client

# start the ZFS backend server part:
# copy the folder ./[web-gui]/data/remote_cs_server to any location on a supported platform ex to /tank/data
# and start cs-backend server (as admin/root) via perl /path_to/remote_cs_server/start_server_as_admin.pl

# Add this server to a socketgroup in the frontend client Windows ZFS gui in menu Extensions > Socketgroup
# the auth key here in /socket/server.auth must be the same as on Windows in /path/_log/groups/hostname~ip.txt


###############################################


use strict;
use vars qw($command $allowed_cmd $allowed_ip $os $fn $on $dn $rh $hn $path $opt $filter $auth $pw $tpath $wpath $dpath $spath @q $ip $t $in %current $client);           # public varlist

use Fcntl qw(:flock);                           # filelocking

# auto-flush on socket
$| = 1;

my $dn = $^O;                                   # distribution from Perl ex MSWin32

# get server path from Perl
$spath=$0;
if (!($spath=~/\/|\\/)) {
  $spath=`cd`;   chomp($spath);
}
$spath=~s/.start_server_as_admin.pl//;          # path to startscript
$spath=~s/\\/\//gs;                             # always use / (even on win with powershell)
print "spath=$spath\n";

#load IPC::Run via require (opt)
# require "$path/CGI/IPC/Run.pm";               # IPC::Run::run(run \@q, \$in, \$out, \$err, timeout( $timeout ));

# read server
my $config=&my_read("$spath/server.pl");        # backend server
my @config=split(/\n/,$config);

# update server.pl
foreach my $l (@config) {   # paths must configured during start of backend server
     if ($l=~/^use lib/)     { $l="use lib \"$spath/CGI\";"; }
     if ($l=~/^\$spath *=/)  { $l="\$spath=\"$spath\";";      }     # ./socket path/tmp or ./web-gui path/tmp
}

$t=join("\n",@config);
&my_write("$spath/server.pl","$t");             # write updated config

#############
# read monitor
my $config=&my_read("$spath/monitor.pl");       # monitor
my @config=split(/\n/,$config);

# update monitor.pl
foreach my $l (@config) {   # paths must configured during start
     if ($l=~/^use lib/)     { $l="use lib \"$spath/CGI\";"; }
     if ($l=~/^\$spath *=/)  { $l="\$spath=\"$spath\";";      }     # ./socket path/tmp or ./web-gui path/tmp
}

$t=join("\n",@config);
&my_write("$spath/monitor.pl","$t");      # write updated config

# on updates, check, opt create cs_server.cfg
$wpath=$spath; $wpath=~s/.cs_server//;             #  ../web-gui/cfg
if (!(-d "/xampp")) {
 if (!(-d "$wpath/cfg")) {           # unix
   print "mkdir $wpath/cfg\n";
  `mkdir $wpath/cfg`;
 }
}


# opt copy /cs_server/server.auth to /cs_server.cfg/server.auth
###########################################################
if ((-f "$spath/server.auth") && (!(-f "$wpath/cfg/server.auth"))) {
   print "cp $spath/server.auth $wpath/cfg/server.auth\n";
   $t=&my_read("$spath/server.auth");
   print &my_write("$wpath/cfg/server.auth",$t);
   unlink("$spath/server.auth");

   #`cp $spath/server.auth $spath.cfg/server.auth`;
}

# start backend server.pl
   #
   if ($dn=~/win32/i) {  # perl not in path, slash allowed in path with powershell
   #`taskkill /IM curl* /F /T`;
   #`taskkill /IM perl* /F /T`;        # cannot kill myself

                         # -Command "$process = Start-Process -FilePath 'C:\path\to\perl.exe' -ArgumentList '-file', 'C:\path\to\perlscript.pl'"
     print "start server and monitor on Win\n";       # win32

     system("powershell -command \"start-process \"C:/xampp/perl/bin/perl.exe $spath/server.pl\"  -WindowStyle minimized\"");
     system("powershell -command \"start-process \"C:/xampp/perl/bin/perl.exe $spath/monitor.pl\" -WindowStyle minimized\"");


   } else {

     # kill prior restart
     `pkill -f cs_server/server.pl`;
     `pkill -f cs_server/monitor.pl`;
     `pkill -f curl`;

     print "start server and monitor under $dn\n";
     system("perl $spath/server.pl &");                                              # asume perl in path
     system("perl $spath/monitor.pl &");
   }



sub my_write {           # write   write("path/filename","data");
##############
    my $file = $_[0];
    my $data = $_[1];

    open(my $fh,">$file");
      flock($fh, LOCK_EX);
        print $fh "$data\n";
      flock($fh, LOCK_UN);
    close($fh);
    return();
}

sub my_read {           # read   read("path/filename");
##############
    my $file = $_[0];

    my $data="";
    my @data=();

    open(my $fh, "<$file");
      flock($fh, LOCK_EX);
         @data = <$fh>;
      flock($fh, LOCK_UN);
    close($fh);
    $data=join("",@data);
    return($data);
}


#  do not edit below
1;
