# remote functions for napp-it cs on Windows
# Copyright (c) 2010 - 2024 napp-it
# functions are autoloaded if action contains &thislib_xx() ex &zfslib_list() will autoload zfslib via require


#########
sub grouplib {  #  # allow remote management
###########################
}


###########  ## log last run in jobfile /_log/jobs/id...job
sub grouplib_remote_log {
###########

  my @p1=caller(0); # this:
  my @p2=caller(1); # parent:  package, filename, line, subroutine, hasargs, wantarray, evaltext, is_require, hints, hitmask
  my @p3=caller(2);

  my $rem="$wpath/_log/remote.log";

  my (@v,$v,$n,$l,$f1,$f2,@data);
  my $info="## $sys{'hour'}:$sys{'min'} $sys{'sec'} grouplib $p1[2]: $_[0]";


  # write
  @data=();
  if (-f $rem) {
         open (F,"<$rem");
               flock(F, LOCK_EX);
               @data=<F>;
         close (F);
  }

  $info=~s/\n/<br>/gs;
  unshift(@data,"$info\n");

  if (@data > 100) {
             $t=@data-100;
             splice(@data,100,$t);
  }

  $t=join("",@data);
  open (PF, ">$rem");
          flock(PF, LOCK_EX);
          print PF "$t";
  close (PF);

}


############################
sub grouplib_ask_remote_sync {   # hide: request remote sync transfer
############################

       my $source_host=$_[0];      # sync from
       if (!($source_host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }


       my @vars=split(/,/, $_[1]); # id,source_ip,target_ip,action,source_zfs

       my $noerr=$_[2];            # no stop on error
       my $ref=$_[3];              # sub called from


       my $id=$vars[0];
       my $source_ip=$vars[1];     # my ip to answer
       my $target_ip=$vars[2];     # my ip to answer
       my $r_action=$vars[3];      # initial, delta, kill
       my $source_zfs=$vars[4];    #

       my $target_host=`hostname`; # hostname not ip
       chomp($target_host);

       #read access key of host
       my $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");


       foreach $t (@t) {
                if (!($t=~/^$source_host\~|~$source_host/)) { next; }


                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $f="action=rgroup\&key=$p[0]&do=request_sync";
                $f.="&hostname=$target_host";
                $f.="&id=$id";
                $f.="&source_ip=$source_ip";
                $f.="&target_ip=$target_ip";
                $f.="&target_host=$target_host";
                $f.="&r_action=$r_action";                         # remote action
                $f.="&source_zfs=$source_zfs";


                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_zfs_sync 46\n $id $i");

                #log
                $t=$r; $t=~s/\n/, /gs;
                $f=~s/.*do/do/;
                print "glib 60 \&grouplib_ask_remote=$f\n\nresult=$r\n";
                &log("jobid $id | glib 61 request remote zfs sync $i:\n from $source_ip $source_zfs to $target_ip\n action=$f \n\nresult $t\n\n","l");

                if ($r=~/error/is) {
                    $r=~s/.*error/error/is;
                    $r="sub ask_remote_zfs_send:<hr>$r";
                } else {
                    $r=~s/.*web-gui-ok://s;
                }

                return ($r);
        }
        return ("not a member");
}

###################################
sub grouplib_replication_job_delete {      # hide: delete replication job
###################################
    # called from autojob

    my ($r,@w,$w,$snap_var);
    require "$dpath/menues/_lib/windows/zfslib.pl";

    $sys{'port'}    = 81;
    my $id          = $_[0];
    my $myhost      = $_[1];
    my $host        = $_[2];
    my $source      = $_[3];
    my $destination = $_[4];
    my $snap_rec    = $_[5];

    # delete source snaps
    $r = &grouplib_ask_remote_snaplist("$host");
    @w = split (/,/,$r);
    foreach $w (@w) {
        if (!($w=~/$source\@$id\_repli_zfs_$myhost\_/)) { next; }
        $w=~s/\ .*//;
        $w=~s/$source\@$id\_repli_zfs_$myhost\_//;
        $snap_var="snap_name=$source\@$id\_repli_zfs_$myhost\_$w\n";
        $snap_var.="snap_rec=$snap_rec\n";
        &grouplib_ask_remote_destroy_snap ("$host", $snap_var);
    }

    # delete destination snaps
    $r = &zfslib_list_datasnaps ("l");
    @w = split (/,/,$r);
    foreach $w (@w) {
        if (!($w=~/$destination\@$id\_repli_zfs_$myhost\_/)) { next };
        $w=~s/\ .*//;
        $w=~s/$destination\@$id\_repli_zfs_$myhost\_//;
        &exe ("zfs destroy $snap_rec $destination\@$id\_repli_zfs_$myhost\_$w");
    }
}


########################
sub grouplib_replication {  # hide:      first time/ incremental replication  from 0.8
#######################
 ###obsolet
 &mess("group_replication: obsolete");  # separate job-script
}

#################################
sub grouplib_start_zfs_receiver {   #  start zfs receiver
#################################
    my (@vars,%vars,$do,$k,$v,$r,$w,@w,$timer,$p,@lr,$lr,$cmd,@s,$s);
    %vars = ();
    @vars=split (/\n/, $_[0]);
    foreach $v (@vars) {
        if (!($v=~/\=/)) { next; }
        chomp($v);
        $k=$v; $k=~s/\=.*//;
        $k=~s/[^a-zA-Z0-9\-\_]//g;
        if ($k ne "") {
            $v=~s/.*\=//;
            $vars{$k} = $v;
        }
    }

    #auto_error_log
       &log("jobid $vars{'id'} | glib grouplib_start_zfs_receiver 550 <br> init receiver $vars{'id'} | zfs receive $vars{'reset_to_snap'} $vars{'dest_zfs'} ","lmp");

   # local transfer # receiver is implemented in remote_call_zfs_send   (netcat locally only ok on Nexenta)
   ##################
   if ($vars{'host'} eq $vars{'myhost'}) {

       $in{'myhost'}=$vars{'myhost'};
       &log("jobid $vars{'id'} < glib grouplib_start_zfs_receiver 550|init local transfer $vars{'id'}","mpl");
       return "OK";
   }
   #### / local


   ## receive options
   ##################
   $p="";
   $timer = time();
   if ($vars{'dest_zfs'}=~/\/\w+/)   {
        # call perlsript post_replication (shares off/ opt on, set ro)
        $p=" \\xampp\\perl\\bin\\perl $dpath/menues/_lib/windows/scripts/post_replication.pl $vars{'id'} $timer || \\xampp\\perl\bin\\perl $dpath/menues/_lib/scripts/post_replication.pl $vars{'id'} error";
   }

  $cmd="$vars{'dest_interface'} | zfs receive $vars{'reset_to_snap'} $vars{'dest_zfs'} >>/tmp/monitor.log 2>&1$p";


   #######
   #run it, (do not restart if running!)
   #######

   # Nexenta NCP and OI/SE11 behaves different
   #######################################

   if ($sys{'auto_os_type'} eq "osol_n") {
       #Nexenta
       $cmd=~s/ -d / /;           # nc -d -l is not allowed, use nc -l only
   }
   $cmd=~s/sudo //g;
   $cmd=~s/ +/ /g;
   #########################################



   #Run via file:
         open (PF, ">/$wpath/_log/tmp/$vars{'id'}\.request_sh");
               print PF "$cmd\n";
         close (PF);


&log("jobid $vars{'id'} | glib grouplib_start_zfs_receiver 599 | console cmd|$cmd","mpl");
sleep (1);

#return "OK";

# wait until receiver is running
################################

my $once="1";

   @lr = split(/\+\+/,$vars{'local_running_ps'});
   my $pscount=@lr;
   $timer = time() + 60;                                   # Wait long enough, especially needed with more than one replication

   do {
        $run = "";
        foreach $lr (@lrdo) {
            $r = &exe("ps axw");
            @w=();
            @w = split(/\n/,$r);

            if ($once) { &log("jobid | $vars{'id'} glib repli 600 \: check $lr","pl"); $once="0"; }
            $r = "";
            foreach $w (@w) {
                if ($w=~/sudo/)   { next; }
                if ($w=~/grep/)   { next; }
                if (!($w=~/$lr/)) { next; }
                $r .= "1";
                $t= time(); $t= $t - ($timer - 60);
                &log("$vars{'id'} | glib repli 608 \: found $w after $t s","pl");

            }
            $run .= $r;
        }
        sleep (2);

   } until ((length($run) == $pscount) or (time() >= $timer));

$t= time(); $t= $t - ($timer - 60);
&log("$vars{'id'}| glib grouplib_start_zfs_receiver 640 <br> start receiver |receiver started after $t s","pl");
#$run=@lr;
#&print_var($run);

   #log
       my $pid=`ps axw`;
                @t=();
                @t=split(/\n/,$pid);
                $pid="";
                foreach $t (@t) {
                   if ($t=~/$p|$vars{'dest_zfs'}/) {
                      $pid.="\n$t";
                   }
                }
                if ($pid eq "") { $pid=" -check local ps for this job; ok, none"; }
                &log("$vars{'id'} | glib repli 633 $vars{'id'}\: \nlocal ps=\n$pid\n\ncmd=\n$cmd\n","pl");


    $t=length($r);
    if ($t < @pscount) {
        &log("$vars{'id'} | glib *** grouplib_start_zfs_receiver 660| could not start receiver","ampl");
        return "Error: could not start receiver!";
    } else {
        &log("$vars{'id'} | glib grouplib_start_zfs_receiver 663|receiver  | receiver started successfully","mpl");
        return "OK";
    }
}

###########################
sub grouplib_source_target {   #  get source and target of a replication
###########################

       # select_source + select_destination selected in menu jobs > create replication
#&print_hash(%in);


       my ($source_fs,$dest_fs,$source_ip,$dest_ip,$source_auth,$dest_auth);
       $source_ip=$in{'select_source'};      $source_ip=~s/.*~//;                  # use ip only
       $dest_ip  =$in{'select_destination'}; $dest_ip  =~s/.*~//;

       $source_auth=$cfg{$source_ip};
       $dest_auth=$cfg{$dest_ip};

#&mess("$source_ip -> $dest_ip<hr>$source_auth , $dest_auth");

       # source_fs
       $source_fs=&socket("zfs list -H -o name auth:=$source_auth",$source_ip,'10');                        # &socket -> cmd auth:=, ip, timeout
#mess($source_fs);
       # dest_fs
       if (source_ip eq $dest_ip) {
         $dest_fs=$source_fs;
       }  else {
         $dest_fs=&socket("zfs list -H -o name auth:=$dest_auth",$dest_ip,'10');

       }

       $source_fs =~s/\n/,/gs;                            # filesystems, commalist
       $dest_fs   =~s/\n/,/gs;

#&print_var($source_fs);

        &ask('begin','b');
        &ask('hidden','sd2','1');                            # source-dest menue 2
        &ask('hidden','dest_filesystems',$dest_fs);          # to check against new name

        print " <b style='background-color:#dddddd'>Replication from: $in{'select_source'}</b><br>";
        &ask('select_source','Source server',$in{'select_source'});
        &ask('select_src_zfs','Source Pool/ Filesystem',$source_fs,$in{'select_src_zfs'});

        print "<br><br><b style='background-color:#dddddd'>Replication to: $in{'select_destination'}</b><br>";
        &ask('select_dest','Destination server',$in{'select_destination'});
        &ask('select_dest_zfs','Destination Pool/ Filesystem<br>replicated filesystem is created below',$dest_fs,$in{'select_dest_zfs'});

        print "it is suggested to disable recursiv and create multiple jobs per filesystem";
        &ask('select_recursive','enable recursiv','no,yes');

        print "Raw send: useful to send encrypted filesystems";
        &ask('select_raw','zfs raw send','no,yes');

        print "Incremental send: only newest data (-i) or include all intermediary snapshots and clones(-I)";
        &ask('select_send_i','zfs send incremental snapshot option','-i,-I');


        ask('value_send_options','other send options',$in{'value_send_options'});
        ask('value_receive_options','other receive options',$in{'value_receive_options'});

        &ask('value_zfs_name','Name of new ZFS (empty=same)',$in{'value_zfs_name'});

        # only needed to recreate job
        &ask('value_force_id','Force jobid (to continue a former job)',$in{'value_force_id'});

        &ask('value_esxi','Include ESXi quiesce/memory snaps for ESXi ip <br>If this FS is a NFS/ESXi datastore',$in{'value_esxi'});

        print "<br> <b style='background-color:#dddddd'>Options:</b><br><br>";
        &ask('select_type','Replication type','zfs send via netcat',$in{'select_type'});


        #keep
        print "<br>";
        $t="hours:n, (hourly snap per last n hours)<br>days|months|years:n, (one per last n days|months|years)<br>ex hours:24,days:32,months:12,years:2";
        &ask('value_keep',"Keep target repli snaps (commalist of values or char)<br>$t",$in{'value_keep'});
        &ask('value_hold','Hold target repli snaps for n (days) or n s (snaps)<br>ex 8 = 8days or 8s = 8 snaps, keep empty to hold last two snaps<br><br>Keep and hold are both respected',$in{'value_hold'});

        print "<br><br><b style='background-color:#dddddd'> Timetable</b><br><br>\n";


        &ask('select_month','Month (1..12)','every','every');
        &ask('select_day','Day (sun..sat 1..31)','every,sun,mon,tue,wed,thu,fri,sat,mon-fri,mon-sat,sat-sun,first-sat,first-sun,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31','sun');
        &ask('select_hour','Hour (0..23)','every,every-2,every-2BC,every-3BC,0,3,6,9,12,15,18,21','0');
        &ask('select_min','Minute','every,every-3,every-5,every-10,every-15,every-20,every-30,0,15,30,45,once','0');
        print "<br>\n";
        &ask('select_status','Status','manual,active','manual');

        print "<a href=\"#\" onclick=\"\$('.info').toggle();\"><img src=\"/_doc/menu/ico/i16.png\"></a>\n";
        print "<div class='info'>Info: 0-3-5 means 0, 3 and 5<br>every-2 (hour) means every two hours<br>every-1BC (hour) means every hour between 7-12 and 13-19 hour (ABCD, each 6 hours from 0..23)";
        print "<br>every month can be every, every-2, every-3,every-4, every-6";
        print "<br>every day can be every, every-2, every-3, every-7, every-14";
        print "<br>every hour can be  every, every-2, every-3, every-4, every-6, every-12, opt followed by ABCD - A=0..5, B=6..11, C=12..17, D=18..23";
        print "<br>every minute can be every, every-3, every-5, every-10, every-15, every-20, every-30";
        print "<br>minute=once means: execute job once after each reboot";
        print "<br><br>Your source/target must have different hostnames and should not change IP";
        print "<br>Your target ZFS is read-only after replication. After disabling replication, you can set it to read/write and use it at usual";
        print "</div><br><br>";

        &ask('end');    # create job , start it manually
        print "<br>Replication is speed optimized and unencrypted. <br>If you want to replicate over unsecure networks like Internet, a VPN link (Wireguard) or encrypted raw send is suggested!<br>\n";




#print_array(@source);


}

##################
sub grouplib_remote_stat {        # hide:
################# basic stats like cap, pool or job error

   #pool stat
   my ($stat,$n,@c,$u,$a,$t,$m,$col);
   my $ant=`sudo $sys{'sbinpath'}zpool list -H -o name`;
   chomp($ant);
   my @p=split(/\n/,$ant);
   my @pools=();

   #first rpool then rest
   if ($sys{'auto_os'} ne "Linux") {
      push(@pools,"rpool");
   }
   @p=sort(@p);

   foreach my $p (@p) {
       if ($p eq "rpool") { next; }
       push(@pools,$p);
   }
   my $pu="";  # p_used
   my $ps="";  # p_stat

   foreach my $p (@pools) {

           # status
           $ant=`sudo $sys{'sbinpath'}zpool status $p`;
           chomp($ant);


           if ($p eq "rpool") {
              if ($ant=~/OFFLINE|DEGRADED|UNAVAIL|FAULTED|REMOVED/s) {
                 $stat.=" r_stat: <img src=\"/_doc/red.png\" title=\"rpool status error\" style='margin-right:10px'> ";
              } else {
                 $stat.=" r_stat: <img src=\"/_doc/green.png\" title=\"rpool status ok\" style='margin-right:10px'> ";
              }
           } else {
                $t="";
                if ($ant=~/OFFLINE/)  { $t="OFFLINE";  }
                if ($ant=~/DEGRADED/) { $t="DEGRADED"; }
                if ($ant=~/UNAVAIL/)  { $t="UNAVAIL";  }
                if ($ant=~/FAULTED/)  { $t="FAULTED";  }
                if ($ant=~/REMOVED/)  { $t="REMOVED";  }

                if ($t ne "") {
                   $ps.="pool $p: $t, ";
                }
           }

           #cap
           $ant=`sudo $sys{'sbinpath'}zfs list -H -o used,avail $p`;
           chomp($ant); $ant=~s/\s+/ /g;
           @c=split(/ /,$ant);
           $u=&grouplib_kb($c[0]);
           $a=&grouplib_kb($c[1]);

           $t=$u/($u+$a); $t=$t*100; $t=sprintf ("%.0f", $t);         # percent used

           # debug
           # $stat.=",p=$p,c0=$c[0],c1=$c[1],u=$u,a=$a,t=$t %";

           if ($p eq "rpool") {
                $col="green.png";
                if ($t>70) { $col="yellow.png"; }
                if ($t>85) { $col="red.png"; }
                $stat.=" r_used: <img src=\"/_doc/$col\" title=\"rpool used=$t%\" style='margin-right:10px'>";
           } else {
                if ($t>$m) { $m=$t; $pu.="pool $p\: $t\%, "; }                      # max pool used
           }
   }

   $ps=~s/, $//;
   if ($ps ne "") {
        $stat.=" p_stat: <img src=\"/_doc/red.png\" title=\"$ps\" style='margin-right:10px'> ";
   } else {
        $stat.=" p_stat: <img src=\"/_doc/green.png\" title=\"datapool status ok\" style='margin-right:10px'> ";
   }


   if ($m>0) {
            $pu=~s/, $//;
            $col="green.png";
            if ($t>70) { $col="yellow.png"; }
            if ($t>85) { $col="red.png"; }
            $stat.=" p_used: <img src=\"/_doc/$col\" title=\"$pu\" style='margin-right:10px'>";
   }

   # joberr
   # jobinfos id.par
        @t=();
        @t=&get_dir('$wpath/_log/jobs/');

        my $tmp="";
        my ($id,$j);
        foreach my $t (@t) {

            # check in id.par if status=active
            if (!($t=~/\.par$/))      { next; }

            $id=$t; $id=~s/\..*//; chomp($id);                         # jobid

            #

            # check if status=active
            $j=&exe("cat $wpath/_log/jobs/$t");
            if (!($j=~/status=active/s))  { next; }                    # job is not active

            # check laststate
            if (!(-f "$wpath/_log/jobs/$id.log"))       { next; }                    # no log
            $j=`type $wpath/_log/jobs/$id.log`;

            #check if log starts with error
            if ($j=~/^error/) {
               $j=~s/\n.*//s;
               $tmp.="$id,";
            }
        }
        if ($tmp eq "") {
             $stat.=" jobs: <img src=\"/_doc/green.png\" title=\"active jobs: no error\" style='margin-right:10px'>";
        } else {
             $stat.=" jobs: <img src=\"/_doc/red.png\" title=\"active jobs with errors\" style='margin-right:10px'>";
        }
    ##
    $stat=~s/^,//;
    return("$stat");

}

sub grouplib_kb {
#################
      my $w1=$_[0]; $w1=~s/ //g;
      if ($w1=~/K/) { $w1=~s/K//;  }
      if ($w1=~/M/) { $w1=~s/M//; $w1=$w1*1000; }
      if ($w1=~/G/) { $w1=~s/G//; $w1=$w1*1000000; }
      if ($w1=~/T/) { $w1=~s/T//; $w1=$w1*1000000000; }
      if ($w1=~/P/) { $w1=~s/P//; $w1=$w1*1000000000000; }
      return ($w1);
}

##################
sub grouplib_remote_data {                # hide:
##################

#&print_hash(%in);
      # action=rgroup_data_abx.txt -> abc.txt is filename
      my $fn=time()."tmp";
      if ($in{'action'}=~/rgroup_data_\w+/) {
         $fn=$in{'action'};
         $fn=~s/rgroup_data_//;
      }

      &my_write("$tpath/$fn","$in{'file'}");

      my $l=length($in{'file'});
      print "Content-type: text/plain\r\nweb-gui-ok:$l Bytes received\n";
      exit;

    #################
}

##################
sub grouplib_remote_call {                # hide:
##################

        #############################################
        # this part is called by another ZFS server
        # to answer remote calls
        #############################################

        &print_hash(%in);

        # actions #########
        # do="" -> return ok:web-guo
        # do=request_membership (pw=adminpw) -> return web-gui-ok:$name=$access (access key from ip+pw)
        #############################################

        # called when:  action=rgroup by sub nasgroup_change_property in nasgroup-lib.pl
        #############

        # other parameter
        # do=        request (request membership)        with ip + pw (admin)
        my ($ver,$status,$r);

       # print $in{'RawData'};
       # &mess($in{'ref'});
         #request socket save data in tpath/filename
        #if ($in{'ref'}=~/\w+/) {                                    #save?filename&data=$data
               # sub my_write {           # write   write("filename","data");
               # unencode data
               # use URI::Escape;

               #my $data_encoded = 'some%20text%2Bwith%20spaces%26symbols';
               #my $data_decoded = uri_unescape($data_encoded);
              # my $data=uri_unescape($in{'data'});

                &my_write("$tpath/aa.data","$in{'RawData'}");
               #&my_write("$tpath/$in{'ref'}","$in{'PlaiNTexT'}");                    # text/plain data separated in ReadParse

               print "Content-type: text/plain\r\nweb-gui-ok:$in{'ref'}\n";
               exit;
       # }

#ask if web-gui?
################
        if ($in{'do'} eq "") {

                #get hostname
                $ver=`hostname`;
                chomp($ver);

                if ($sys{'version'} eq "") {
                    $ver="$ver v. n.a.";
                } else {
                    $ver="$ver v. $sys{'version'}";
                }


                #add ext
                if ($cfg{'ver'} ne "") {
                     $ver.=" ,$cfg{'ver'}\t";

                     if ($cfg{'ver'}=~/c|m/) {
                          #stat
                          $ver.=&grouplib_remote_stat;
                     }
                } else {
                     $ver.="\t p_stat: <img src=\"/_doc/grey.png\" title=\"p_stat\" style='margin-right:10px'>";
                     $ver.=" r_stat: <img src=\"/_doc/grey.png\" title=\"r_stat\" style='margin-right:10px'>";
                     $ver.=" r_used: <img src=\"/_doc/grey.png\" title=\"r_used\" style='margin-right:10px'>";
                     $ver.=" p_used: <img src=\"/_doc/grey.png\" title=\"p_used\" style='margin-right:10px'>";
                     $ver.=" jobs: <img src=\"/_doc/grey.png\" title=\"jobs\" style='margin-right:10px'>";
                }

                #get/ return basic status
                $status=$ver;                                # was ok$ver
                &log("noid| grouplib 885 remote call | status=<br> $in{'do'}","m");

                print "Content-type: text/plain\r\nweb-gui-ok:$status";
                exit;
        }  else {
                &log("noid| grouplib 889 remote call | do=<br> $in{'do'}","m");
        }

        #log##
       # &log("grouplib_remote_call 838|||$in{'do'}","lm");                #opt


# request sync
##############
        if ($in{'do'} eq "request_sync") {
            &grouplib_remote_call_sync;
            exit;
        }

#request membership from other system?
######################################
        if ($in{'do'} eq "request_membership") {
            &grouplib_remote_call_membership;
            exit;
        }
#request pslist?
################# with key but only form host from /group/filename

        if ($in{'do'} eq "request_pslist") {
            &grouplib_remote_call_pslist;
            exit;
        }
#request zfslist?
################# with key but only form host from /group/filename

        if ($in{'do'} eq "request_zfslist") {
            &grouplib_remote_call_zfslist;
            exit;
        }
#request zfs_send?
##################
        if ($in{'do'} eq "request_zfs_send") {
            &grouplib_remote_call_zfs_send;
            exit;
        }
#request get_my_ip?
###################
        if ($in{'do'} eq "request_get_my_ip") {
            &grouplib_remote_call_get_my_ip;
            exit;
        }
#request destroy_snap?
######################
        if ($in{'do'} eq "request_destroy_snap") {
            &grouplib_remote_call_destroy_snap;
            exit;
        }
#request destroy_proc?
######################
        if ($in{'do'} eq "request_destroy_proc") {
            &grouplib_remote_call_destroy_proc;
            exit;
        }

        exit;
}


######################################
#request membership from other system?
######################################

##################
sub grouplib_remote_call_membership {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z);


        $from=$ENV{'REMOTE_ADDR'};
        $name=`hostname`;
        chomp($name);

        #pw ok?
        if ($cfg{'value_adminpw'} ne "") {
           $pass=&gethash("admin",$in{'pw'},"0");

           if ($cfg{'value_adminpw'} ne "$pass" ) {
              print "Content-type: text/plain\r\nok:request_membership:wrong admin password for $name";
              exit;
           }
        }

        #pw ok -> return accesspw bound to ip
        $access=&gethash("ad",$from,"0");
        $access.=&gethash("ad",$in{'pw'},"0");


        #write access key to /group/hostname~ip.txt
        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        &str2file("$access","$file");

        print "Content-type: text/plain\r\nweb-gui-ok:$name=$access";

         &log("noid |  glib grouplib_remote_call_membership 923|request membership $from||ok $name","mr");

        exit;
}




#request zfslist?
################# with key but only form host from /group/filename

##################
sub grouplib_remote_call_pslist {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z,$t);
        $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        #get ps list
                $r=`sudo ps axw`;

                @t=();
                @t=split(/\n/,$r);
                $r="";
                $t="";
                foreach $z (@t) {
                          $z=~s/^ +//;
                          if ($z=~/^PID/)              { next; }
                          $z=~s/ .*\d:\d\d /  /;        # pid command
                          $z=~s/\;\;//g;

                          # only zfs, mbuffer,rsync,cp,nc
                          if (!($z=~/zfs |mbuffer |rsync |\bcp |\bnc /))  { next; }
                          $r.="$z".";;";           # trennzeichen
                          $t.="$z\n";
                }

               #opt. log to replicate-log
               #open (PF, ">>/var/web-gui/data/napp-it/_log/last_replication.log");
               #   print PF "\n\n###sub grouplib_remote_call_zfs_pslist_return value###\n$r\n";
               #close (PF);


        $r=~s/;+$//;
        print "Content-type: text/plain\r\nweb-gui-ok:$r";

        &log("noid | glib grouplib_remote_call_pslist 986|request pslist $from||$t","mr");
        exit;
}

##################
sub grouplib_remote_call_zfslist {                # hide:         answer
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z,$t);
        $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        #get zfs/ snap list
                if ($in{'type'}=~/snap/) {
                                $r=&exe("zfs list -r -o name,used,avail,refer -t snapshot");
                } else {
                                $r=&exe("zfs list -H -t filesystem,volume -o name");
                }
                @t=();
                @t=split(/\n/,$r);
                $r="";
                $t="";
                foreach $z (@t) {
                           if ($z=~/^syspool\b/)              { next; }
                           if ($z=~/^rpool\b/)                { next; }
                           if ($z=~/^NAME\s/)                 { next; }
                           $r.="$z,";
                           $t.="$z\n";
                }
        $r=~s/,+$//;
        print "Content-type: text/plain\r\nweb-gui-ok:$r";

        &log("noid | glib 1067 <br> grouplib_remote_call_zfslist|request zfslist ($in{'type'}\) $from|||$t","mr");
        exit;
}


##################
sub grouplib_remote_call_sync {                # hide:
##################

        my $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        my $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        if ($in{'id'} eq "") {
             print "Content-type: text/plain\r\nweb-gui-ok:id is missing"; exit;
        }

        ############
        # part 1 create snap
        ############

        # delete this snap if already exist due to an former error
        # zfs@job_sync_repli_target_host

        my $name="$in{'id'}"."_sync";
        $name=~s/^\/+//;        # pool/zfs

        my $r=`sudo zfs list -t snapshot -H -o name | grep $name`;

        $name="$in{'source_zfs'}\@$in{'id'}"."_sync_repli_$in{'target_host'}";
        $name=~s/^\/+//;        # pool/zfs

        if ($r=~/\@/) {
           &log("jobid $in{'id'} | glib 1115 <br> request snapdel | del old dnap <br> $name","mr");
           $r=`sudo zfs destroy -f $name`;
        }

        $r=`sudo zfs snapshot $name`;       # crete new snap
        &log("jobid $in{'id'} | glib 1121 created snap<br> zfs snapshot | create snap <br> $name <br>$r","mr");

        ###########################
        # part 2 start initial sync
        ###########################   source: $s="$source_zfs/.zfs/snapshot/$job\_sync_repli_$target_host";

        my $port=&port_from_id($in{'id'});

        my $s="/$in{'source_zfs'}/.zfs/snapshot/$in{'id'}\_sync_repli_$in{'target_host'}";
        $s=~s/^\/+/\//;

        # log folder
        $t=`sudo ls $s/.`;
        $t=substr($t,0,200); $t=~s/\n/<br>/gs; $t.=" ...";

        &log("jobid $in{'id'} | glib 1134 folders in <br>$s  | $t","mr");


        my $cmd="cd $s\n tar cpf - ./ | nc $in{'target_ip'} $port\n";
        $cmd.=" del $wpath/_log/jobs/$in{'id'}\~remote~initiated.job";


         # create job in joblist
         `eco "" > $wpath/_log/jobs/$in{'id'}\~remote~initiated.job`;


        #1. optional, delete running send job (must be an error)

            $r = `ps axw | grep nc.*$port`;
            if ($r ne "") {
                 @t=();
                 @t=split(/\n/,$r);
                 foreach my $t (@t) {
                    $t=~s/^\s+//;
                    $t=~s/ .*//; # keep nr
                    print "old process: kill -9 $t\n";
                    `sudo kill -9 $t`;
                 }
            }


         # 2. Run via file and background agents:
         open (PF, ">$wpath/_log/tmp/$in{'id'}\.request_sh");
               print PF "#!/bin/sh\n";
               print PF "$cmd\n";
         close (PF);

     #debig
     # `sudo cp /root/a.sh /var/web-gui/data/napp-it/_log/tmp/1338226601.request_sh`;
     #    $r=`sudo nohup sh /tmp/$in{'id'}\_sync_request.sh &`;
     #   $r=`sudo nohup sh /tmp/a.sh &`;


      # start sender once
         $cmd=~s/\|/I/gs;
         &log("jobid $in{'id'} | glib 1166<br> try to start sync sender jobid<br>wait 50s<br>$r  | $cmd","mlr");
         &log("jobid $in{'id'} | glib 1168 <br> result<br>$r","ml");


      sleep(5);

      $r="";
      #check for nc .. port
      my $timer = time() + 50;
      do {
         $r = `ps axw | grep nc.*$port`;
         @t=();
         @t=split(/\n/,$r);
         $r="";
         foreach $t (@t) {
           if ($t=~/grep/) { next; }
           $r.="$t\n"
         }
         sleep (2);
      } until (($r ne "") or (time() >= $timer));

      # to debug, start receiver then sender script manually
      #if (-f "/var/web-gui/data/napp-it/_log/sender.sh") {
      #     print "Content-type: text/plain\r\nweb-gui-ok:";
      #     exit;
      #}
      #/debug
      #######

       print "Content-type: text/plain\r\nweb-gui-ok:";               # no return value

       if ($r ne "") {
          $r.= `ps axw | grep tar `;
          &log("jobid $in{'id'} | glib 1193<br> sync sender  started: ok | $r","mlr");
       }  else {
          &log("jobid $in{'id'} | glib 1195<br> start sync sender jobid $in{'id'} failed | process nc.*$port<br> not found <br>error: sender not started","mlpra");
       }

}




#request zfs_send?
##################
##################
sub grouplib_remote_call_zfs_send {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z,$cmd,$p,$timer);

# log
#####

## basics
#########

        $from=$ENV{'REMOTE_ADDR'};

        &grouplib_remote_log("remote call zfs send $in{'id'} initiated<br>_________________________________________________________<br>");

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

############
#part 1 create snap
############
        # delete this snap if already exist due to an former error
        $r=&exe("zfs list -t snapshot | grep $in{'snap_name2'}");
        if ($r=~/\@/) {
           &log("noid | glib grouplib_remote_call_zfs_send 1286<br>request snapdel $from|||delete $in{'snap_name2'}","m");
           $r=&exe("zfs destroy -f $in{'snap_name2'}");
        }

        # optional del snap
        $r=&exe("zfs snapshot $in{'snap_rec'} $in{'snap_name2'}");
        &log("noid | glib grouplib_remote_call_zfs_send 1291<br>request snapdel $from||zfs snapshot $in{'snap_rec'} $in{'snap_name2'}","m");


 #init send job
       # $cmd="sudo zfs send $in{'send_dedup'} $in{'send_rec'} $in{'send_inc'} $in{'snap_name1'} $in{'snap_name2'} | $in{'src_interface'} >> /tmp/monitor.log 2>&1$t";

        if (!($in{'id'} > 0)) { $in{'id'}=time(); }

        #parameter
        open (PF, ">/tmp/nappit/job-repli-send.$in{'id'}");         # e
                       flock(PF, LOCK_EX);
                       print PF "id=$in{'id'}\n";
                       print PF "send_dedup=$in{'send_dedup'}\n";
                       print PF "send_rec=$in{'send_rec'}\n";
                       print PF "send_inc=$in{'send_inc'}\n";
                       print PF "snap_name1=$in{'snap_name1'}\n";
                       print PF "snap_name2=$in{'snap_name2'}\n";
                       print PF "src_interface=$in{'src_interface'}\n";

                       print PF "send_ip=$in{'port'}\n";
                       print PF "port=$in{'port'}\n";
                       print PF "transport=$in{'transport'}\n";
                       print PF "buffer=$in{'buffer'}\n";
                       print PF "send_i=$in{'send_i'}\n";
        close (PF);

        #request, must be done via at now to allow multiple requests at the same time
        #open (PF, ">/tmp/nappit/job-repli-send.request");          #
        #               flock(PF, LOCK_EX);
        #               print PF "remote_$in{'id'} $time";
        #close (PF);

        `echo \'\\xampp\\perl\\bin\\perl $dpath/menues/_lib/scripts/job-repli-send.pl remote_$in{'id'}\' > /tmp/nappit/repli-send_$in{'id'}.sh`;

        #$r=`sudo /usr/bin/at  -f /tmp/nappit/repli-send_$in{'id'}.sh now`;            # request send via at now

        # or via taskserver
        `echo \'/xampp/perl/bin/perl $dpath/menues/_lib/scripts/job-repli-send.pl remote_$in{'id'}\' > /var/web-gui/_log/tmp/now_$in{'id'}.sh`;

        &grouplib_remote_log("call job-repli-send.pl for id $in{'id'} on interface $in{'src_interface'}<br>$r");

        `del /tmp/nappit/repli-send_$in{'id'}.sh`;


 ####
 exit;














#################
# part 2a # local transfer # receiver is implemented in remote_call_zfs_send   (netcat locally only ok on Nexenta)
#################
if (($in{'host'} eq $in{'myhost'}) && ($in{'host'} ne "")) {

          #get id from snapname
          $id=$in{'snap_name1'};    # ex tank@1306827418_repli_zfs_se11-dev_nr_1'
          $id=~s/_.*//;
          $id=~s/.*\@//;
#test
#&print_var($id);
#&print_hash(%in);

         #log
         &log("noid | glib grouplib_remote_call_zfs_send 1095 <br> request send to localhost<br>host: $in{'host'}<br> myhost: $in{'myhost'}|||zfs send $in{'id'} local transfer:\nsource: $in{'src_interface'}\ndest: $in{'dest_zfs'}\nshare_name:$in{'dest_sharename'}\nshare_guest:$in{'dest_shareguest'}\nreset_to_snap: $in{'reset_to_snap'}","mlr");

        ## receive options
        $p="";
        $timer = time();
        if ($in{'dest_zfs'}=~/\/\w+/)   {
             # call perlsript post_replication (shares off/ opt on, set ro)
             $p=" \&\& sudo perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/post_replication.pl $in{'id'} $timer \|\| sudo perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/post_replication.pl $in{'id'} error";
        }

       #   $cmd="zfs receive $in{'reset_to_snap'} $vars{'in_zfs'} 2>&1 >> /var/web-gui/data/napp-it/_log/last_replication.log $p";
       $cmd="sudo zfs send $in{'send_dedup'} $in{'send_rec'} $in{'send_inc'} $in{'snap_name1'} $in{'snap_name2'} | sudo zfs receive $in{'reset_to_snap'} $in{'dest_zfs'} >> /tmp/monitor.log 2>&1 $p";

}   else {

    ###############
    #part 2b, remote
    ###############


        #Option 1: Geht bei Nexenta, nicht aber bei OI
        #$r=`echo \'sudo zfs send $in{'send_rec'} $in{'send_inc'} $in{'snap_name1'} $in{'snap_name2'} | $in{'src_interface'}\' |  at -s now`;    #ok bei Nexenta
        $t=" && echo '++++++ zfs send finished:' >> /var/web-gui/data/napp-it/_log/current_remote.log; date >> /var/web-gui/data/napp-it/_log/current_remote.log;";
        $cmd="sudo zfs send $in{'send_dedup'} $in{'send_rec'} $in{'send_inc'} $in{'snap_name1'} $in{'snap_name2'} | $in{'src_interface'} >> /tmp/monitor.log 2>&1$t";

}

#############
#### run send
#############

       $cmd=~s/ +/ /g;
       #$cmd=~s/sudo //g;

      # check if there is already a job sending, zfs send .. | nc -w 10 172.16.11.101 53996
      $p=$in{'src_interface'};
      $p=~s/\s+$//;
      $p=~s/^.* //;                                # result: port

      #  #Run via file:
        `touch /tmp/replication_send_$p\.sh`;
         `sudo chmod 755 /tmp/replication_send_$p\.sh`;
         open (PF, ">/tmp/replication_send_$p\.sh");
               print PF "#!/bin/sh\n";
               print PF "$cmd\n";
         close (PF);

      &log("noid | glib grouplib_remote_call_zfs_send 1150 <br>request send to $from via <br>/tmp/replication_send_$p\.sh|console command|$cmd","lmr");



      sleep(3);
      if ($p ne "") {  $r = &exe("ps axw | grep $r"); }
      #check for nc .. port
      @w = split(/\n/,$r);
      $r="0";
      foreach $w (@w) {
        if ($w=~/nc .*$p/) { $r="1";  }
      }

      # to debug, start receiver then sender script manually
      #if (-f "/var/web-gui/data/napp-it/_log/sender.sh") {
      #     print "Content-type: text/plain\r\nweb-gui-ok:";
      #     exit;
      #}
      #/debug
      #######

       # start sender once
       ####################
      # &mess($p);

      # return
       print "Content-type: text/plain\r\nweb-gui-ok:";               # no return value

       if ($r eq "0") {
              #     $r=system("sudo perl -e \'`at -s -f /tmp/replication_send_$p\.sh now > /dev/null 2>&1`\'");           #ok
              #     $r=system("sudo perl -e \'`at -s -f /tmp/replication_send_$p\.sh now`\'");           #ok
            &log("noid | glib start zfs send 1189 <br> sh /tmp/replication_send_$p\.sh|sender started|| $in{'snap_name1'} $in{'snap_name2'}","mlr");
            `sudo nohup sh /tmp/replication_send_$p\.sh &`;

       }  else {
            &log("noid | glib start zfs send 1191 <br>port $p unavailable to send|||error: sender not started","mlpr");
       }
       exit;
}


#request get my ip?
###################

##################
sub grouplib_remote_call_get_my_ip {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z);
        $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        print "Content-type: text/plain\r\nweb-gui-ok:$from";
        exit;
}


#request destroy_snap?
######################

##################
sub grouplib_remote_call_destroy_proc {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z,$r,$t);
        $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        $r=&exe("ps axw");
        @n=split(/\n/,$r);
        if ($in{'kill_rproc'}=~/zfs send|nc /) {
           foreach $z (@n) {
                 if ($z=~/$in{'kill_rproc'}/) {
                      $z=~s/^\s+//;
                      $z=~s/ .*//;
                      $z.=`sudo kill -9 $z`;                  #id
                      $t.="kill $z\n";
                 }
           }
        }

        print "Content-type: text/plain\r\nweb-gui-ok:$r";

        &log("noid | glib grouplib_remote_call_destroy_proc 1274 <br>request kill $from|||$t","mr");                #opt

        exit;
}

##################
sub grouplib_remote_call_destroy_snap {                # hide:
##################
my ($from,$pass,$access,$file,$name,$status,$key,$id,@n,$z,$t);
        $from=$ENV{'REMOTE_ADDR'};

        #key ok?
        if ($in{'key'} eq "") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is missing"; exit;
        }

        $file="$wpath/_log/group/$in{'hostname'}\~$from\.txt";
        if (!(-f $file)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:error this host $from is not a member"; exit;
        }
        &read_ahfile($file,"t");
        if ($t[0] ne "$in{'key'}") {
                   print "Content-type: text/plain\r\nweb-gui-ok:key is wrong"; exit;
        }

        if (!($file=~/$from/)) {
                   print "Content-type: text/plain\r\nweb-gui-ok:not allowed for ip $from"; exit;
        }

        $r=&exe("zfs destroy $in{'snap_rec'} $in{'snap_name'}");

#         $r="\n";
#         $r.="zfs destroy $in{'snap_rec'} $in{'snap_name'}";

        print "Content-type: text/plain\r\nweb-gui-ok:$r";

         &log("glib grouplib_remote_call_destroy_snap 1311|request destroy snap $from|||zfs destroy $in{'snap_rec'} $in{'snap_name'}\n$r","mr");                #opt

        exit;
}

######################
sub grouplib_ask_remote_pslist {                #hide: parameter host, return commalist of ps - id + command (if a nas-member)
######################
        my ($host,$t,$f,$i,$n,$noerr,$ref);
        $host=$_[0];
        $noerr=$_[1];
        $ref=$_[3];

        if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

       foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_pslist&hostname=$n";

                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_pslist 1332");

                if ($noerr eq "") {
                  if ($r=~/error/is) {
                       $r=~s/.*error/error/is;
                       &mess("sub ask_remote_pslist $i:<hr>$r");
                  }
                }

                $r=~s/.*web-gui-ok://s;

                return ($r);
        }
        if ($host eq "localhost") {
           $r=&exe("ps axw");
           $r=~s/\n/;;/gs;
           return ($r);
# &mess($r);
        }
        return ("not a member");
}

######################
sub grouplib_ask_remote_zfslist {                #hide: parameter host, return commalist of zfs (if a nas-member)
######################
        my ($host,$t,$f,$i,$n,$noerr,$ref);
        $host=$_[0];
        $noerr=$_[1];
        $ref=$_[3];

        if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;
                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_zfslist&hostname=$n";
                $f.="&host_ip=$host";
print "\n100 ask\n";
                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_zfslist 1379");
print "$r\n";
                if ($noerr eq "") {
                  if ($r=~/error/is) {
                       $r=~s/.*error/error/is;
                       &mess("sub ask_remote_zfslist $i:<hr>$r");
                  }
                }
                $r=~s/.*web-gui-ok://s;
                return ($r);
        }
        return ("not a member");
}

######################
sub grouplib_ask_remote_snaplist {                #hide: parameter host, return commalist of zfs (if a nas-member)
######################
        my ($host,$t,$f,$i,$n,$noerr,$ref);
        $host=$_[0];
        $noerr=$_[1];
        $ref=$_[3];

        if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;
                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_zfslist&hostname=$n&type=snapshots";
                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_snaplist 1416");
                if ($noerr eq "") {
                  if ($r=~/error/is) {
                       $r=~s/.*error/error/is;
                       &mess("sub ask_remote_snaplist $i:<hr>$r");
                  }
                }
                $r=~s/.*web-gui-ok://s;
                return ($r);
        }
        return ("not a member");
}

######################
sub grouplib_ask_remote_zfs_send {                #hide: parameter host,
######################
        my ($host,@vars,%vars,$k,$v,$t,$f,$i,$n,$noerr,$rh,$ref);
        %vars = ();
        $host=$_[0];
       if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

        @vars=split (/\n/, $_[1]);
        foreach $v (@vars) {
            if (!($v=~/\=/)) { next; }
            chomp($v);
            $k=$v; $k=~s/\=.*//;
            $k=~s/[^a-zA-Z0-9\-\_]//g;
            if ($k ne "") {
                $v=~s/.*\=//;
                $vars{$k} = $v;
            }
        }
        $noerr=$_[2];
        $ref=$_[3];
#test
#&print_hash(%vars);


       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_zfs_send&hostname=$n";
                $f.="&snap_name1=$vars{'snap_name1'}";
                $f.="&snap_name2=$vars{'snap_name2'}";
                $f.="&snap_rec=$vars{'snap_rec'}";
                $f.="&send_rec=$vars{'send_rec'}";
                $f.="&send_inc=$vars{'send_inc'}";               # -i
                $f.="&send_i=$vars{'send_i'}";                   # new parameter in 0.9 send_rec=-i/-I

                $f.="&send_dedup=$vars{'send_dedup'}";
                $f.="&src_interface=$vars{'src_interface'}";

                #needed locally
                $f.="&dest_zfs=$vars{'dest_zfs'}";
                $f.="&dest_shareguest=$vars{'dest_shareguest'}";
                $f.="&dest_sharename=$vars{'dest_sharename'}";
                $f.="&reset_to_snap=$vars{'reset_to_snap'}";
                $f.="&id=$vars{'id'}";
                $f.="&host=$vars{'host'}";
                $f.="&myhost=$vars{'myhost'}";

                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_zfs_send 1483\n $n $i");

                #log
                $t=$r; $t=~s/\n/, /gs;
                &log("noid | glib 1455 request remote zfs send $i:\n sender snap $vars{'id'}\: zfs snapshot $vars{'snap_rec'} $vars{'snap_name2'}\n sender $vars{'id'} : $host\: zfs send $vars{'send_dedup'} $vars{'send_rec'} $vars{'send_inc'} $vars{'snap_name1'} $vars{'snap_name2'}\n result: $t","l");

                if ($r=~/error/is) {
                    $r=~s/.*error/error/is;
                    $r="sub ask_remote_zfs_send:<hr>$r";
                } else {
                    $r=~s/.*web-gui-ok://s;
                }

                return ($r);
        }
        return ("not a member");
}


######################
sub grouplib_ask_remote_get_my_ip {                #hide: parameter host, return my ip
######################
        my ($host,$noerr);
        $host=$_[0];
        $noerr=$_[1];

        if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

        #read access key of host
        $f="$wpath/_log/group";
        @t=();
        &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_get_my_ip&hostname=$n";

                $r=&grouplib_ask_remote("$f","$i","$noerr");

                if ($r=~/error/is) { $r=~s/.*error/error/is; &mess("sub ask_remote_get_my_ip $n:<hr>$r"); }
                $r=~s/.*web-gui-ok://s;
                return ($r);
        }
        return ("not a member");
}

######################
sub grouplib_ask_remote_destroy_proc {                #hide: parameter host, proc
######################
        my ($host,@vars,%vars,$k,$v,$t,$f,$i,$n,$noerr,$proc,$ref);
        $host=$_[0];
        $proc=$_[1];
        $noerr=$_[2];
        $ref=$_[3];

       if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }

       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_destroy_proc&hostname=$n";
                $f.="&kill_rproc=$proc";

                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_destroy_proc");
                if ($r=~/error/is) {
                    $r=~s/.*error/error/is;
                    $r="sub ask_remote_zfs_send:<hr>$r";
                } else {
                    $r=~s/.*web-gui-ok://s;
                }
                return ($r);
        }
        return ("not a member");
}

######################
sub grouplib_ask_remote_destroy_snap {                #hide: parameter host, return my ip
######################
        my ($host,@vars,%vars,$k,$v,$t,$f,$i,$n,$noerr,$ref);
        %vars = ();
        $host=$_[0];
        @vars=split (/\n/, $_[1]);
        foreach $v (@vars) {
            if (!($v=~/\=/)) { next; }
            chomp($v);
            $k=$v; $k=~s/\=.*//;
            $k=~s/[^a-zA-Z0-9\-\_]//g;
            if ($k ne "") {
                $v=~s/.*\=//;
                $vars{$k} = $v;
            }
        }
        $noerr=$_[2];
        $ref=$_[3];

        if (!($host=~/\w+|\d+\.\d+\.\d+\.\d+/)) { return('need hostname or ip'); }


       #read access key of host
       $f="$wpath/_log/group";
       @t=();
       &read_dir("$f","t");

        foreach $t (@t) {
                if (!($t=~/^$host\~|~$host/)) { next; }

                # read access key
                $f="$wpath/_log/group/$t";
                &read_ahfile($f,"p");
                $i=$t; $i=~s/\.txt$//; $i=~s/.*\~//;

                $n=`hostname`;
                chomp($n);

                $f="action=rgroup\&key=$p[0]&do=request_destroy_snap&hostname=$n";
                $vars{'snap_name'}=~s/ .*//;
                $f.="&snap_name=$vars{'snap_name'}";
                $f.="&snap_rec=$vars{'snap_rec'}";

#&print_var("$f -> $i");

                $r=&grouplib_ask_remote("$f","$i","$noerr","$ref -> grouplib_ask_remote_destroy_snap");
                if ($r=~/error/is) {
                    $r=~s/.*error/error/is;
                    $r="sub ask_remote_zfs_send:<hr>$r";
                } else {
                    $r=~s/.*web-gui-ok://s;
                }
                return ($r);
        }
        return ("not a member");
}


###############
sub grouplib_ask_remote {      # hide:  ask remote group via post and netcat/socat, return answer
###############
#parameter:         0=data (encoded) -> a-z A-Z 0-9 ,._;:#+- allowed
#                1=ip

# connect a remote napp-it via netcat (opt socat) on port  = 81 per default to request a remote function
# data are transfered via post to hide them from logs (pw etc)
# -> connect to sub remote_call (see above)

#        POST /cgi-bin/program.pl HTTP/1.0\r\n
#        Accept: www/source\r\n
#        Accept: text/html\r\n
#        Accept: text/plain\r\n
#        User-Agent: Lynx/2.4 libwww/2.14\r\n
#        Content-type: application/x-www-form-urlencoded\r\n
#        Content-length: 35\r\n\r\n
#        user=Larry%20Bird&age=35&pass=testing                                        #urlencoded data!!
##############################################


# example get html page via netcat/socat
#        $ socat - TCP:localhost:$sys{'port'}
#          netcat $sys{'port'}
#        GET /
#        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
#        <html>
#        ...
##############################################
#$data=~ s/([^A-Za-z0-9\=\;\&\_\-])/sprintf("%%%02X", ord($1))/seg;                  #url-encode
#$str =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;                                #url-decode

        my ($ask,$l,$data,$ip,@w,$host,$noerr,$ref,$r,$ncwait);
        $data=$_[0];
        $ip=$_[1];
        $noerr=$_[2];
        $ref=$_[3];        # debuginfo called from


        print "1718 ping -n 1 -W 3000 $ip\n";                                # not needed with ::LWP
        $r=&exe("ping -n 1 -W 3000 $ip");#print "answer:\n$r";


        if ($r=~/unreach/s) { #
           if ($noerr eq "") {
                print "   *** error: $ip offline\n";
                return ("error: $ip offline");
            } else {
                return;
            }
        }

        #netcat wait from about > settings
        $ncwait=120;
        #$t=`cat /var/web-gui/_log/napp-it.cfg | grep nc_wait`;      chomp($t);
        $t=~s/\D//gs;
        if ($t>10) {
          $ncwait=$t;
          $ncwait=$cfg{'value_nc_wait'};
          &log("noid | glib 1749 | repli ask remote host via nc $ip $sys{'port'}|ncwait=$ncwait","lm");
        }

       @w=split(/\&/,$data);
        foreach $t (@w) {
           $t=~ s/([^A-Za-z0-9\=\,\.\-\_\/])/sprintf("%%%02X", ord($1))/seg;
        }

        $data=join("&amp;",@w);
        $l=length($data);

        $ask ="POST /cgi-bin/admin.pl HTTP/1.0\r\n";
        $ask.="Accept: www/source\r\n";
        $ask.="Accept: text/html\r\n";
        $ask.="Accept: text/plain\r\n";
        $ask.="Content-type: application/x-www-form-urlencoded\r\n";
        $ask.="Content-length: $l\r\n\r\n";
        $ask.= $data;

        #connect via netcat/ socat on port $sys{'port'}
        $ask="'print \""."$ask"."\";'";
        #############################


        if (!($sys{'port'} > 1)) {  $sys{'port'}="81"; }                   # default
        $r=&exe("$perl -e $ask | $nc -w $ncwait $ip $sys{'port'}");        # timeout after ncwait s

#&mess($r);
        if (($r eq "") && ($noerr eq ""))  { $r="timeout"; }

        $ask=~s/.*Content-length: $l\r\n\r\n//s;
        $w=$r; $w=~s/\n/, /gs;

        $t=$ask; $t=~s/.*do=/do=/; $t=~s/\".*//;  $t=~s/\%2F/\//g; $t=~s/\%20/ /g; $t=~s/\%40/\@/g;                         # short monitorlog
        my $answer=$w; $answer=~s/.*web-gui-ok://s;
        $host=$t;
        if ($host=~/hostname/) { $host=~s/.*hos

        tname=//; $host=~s/\&.*//; } else { $host=""; }
        &log("noid | glib 942 | repli ask remote host $ip |$ref <br>nc -w $ncwait $ip $sys{'port'}\n$t|answer=<br>$answer","lm");           # log only

        #debug only
        #print "\nremote ask - answer:$r\n\n";

        return ($r);
}



#######################
 sub grouplib_nasgroup_overview {            # print group overview
 ############### print overview if user zfs
    my ($group,@z,$d,$url,$u,$n,$r,@w,@g,$w,$out,$ask,$host,$s,$z,$p,$count);
    $count="0";
    #set property -> answered
            if ($in{'answered'}) { &grouplib_change_property; }

    #delete -> answered
            if ($in{'action'} eq "del-nasgroupmember") { &grouplib_change_property; }

        #delete -> answered
            if ($in{'action'} eq "add-nasgroupmember") { &grouplib_ask_property; }

    #url
    $url="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;member=$in{'member'}&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;type==type=\&amp;action==action=\&amp;list==list=\&amp;prop==prop=\&amp;old==old=\"=ask=>=text=</a>";
    $ask=" onclick=\"return(confirm(\'=ask2=?\'))\"";

    ##############################
    #read list of groups-members
    $t="$wpath/_log/group";
    @t=();
    &read_dir("$t","t");

    foreach $t (@t) {
                if (!($t=~/~\d+\.\d+\.\d+\.\d+\.txt/)) { next; }               # no group member
                if ($t=~/localhost~127.0.0.1.txt/)     { next; }               # always first

                $t=~s/\.txt$//;
                $t=~s/ +//g;                                                   # in case of spaces in membername
                push (@g,"$t");                                                # member~ip
    }

    @g=sort(@g);
    unshift (@g,'localhost~127.0.0.1');                                        # always first, other sorted

#test
#&print_array(@g); # list of members;

    my $member;

    foreach my $g (@g) {
        @w=();
        @w=split(/\~/,$g);

        if (!($w[1]=~/\d+\./)) { next; }                #  ip

        $member="$g";
        #&mess($member);

        #ask remote hostname
        $r=&socket("rh auth:=$cfg{$current{'member_ip'}}","$w[1]","3");  #ask,ip,timeout, do not &mess on errors
        &log_add($r);

#&print_hash(%current);
#print "$g -> $r<br>auth:=$cfg{$current{'member_ip'}}";
#&mess("$g -> $r<hr>$cfg{$current{'$member_ip'}}");

        #parse r
        if ($r=~/auth error/s) {
           $r="authentication error";
        } else {
           if (!($r=~/\w+:\w+/)) {  $r="off, ip forbidden or timeout"; }  #
        }
        $g.="\t$r";                    # add online state/hostname

        # groupname -> link. show members (details) -> "prop"
        # +ZFS,SNAP.PID
        ################

        $z=$url;                       # for zfs list
        $s=$url;                       # for snaplist
        $p=$url;                       # for pslist

        ## ZFS, Snaps and Processes
        $z=~s/=text=/ZFS /;
        $z=~s/=list=/details/;
        $z=~s/=prop=/$member/;
        $z=~s/=action=/remote-details/;
        $z=~s/=type=/zfs/;
        $z=~s/=old=//;
        $z=~s/=ask=//;

        $s=~s/=text=/Snaps /;
        $s=~s/=list=/details/;
        $s=~s/=prop=/member/;
        $s=~s/=action=/remote-details/;
        $s=~s/=type=/snapshot/;
        $s=~s/=old=//;
        $s=~s/=ask=//;

        $p=~s/=text=/Pid /;
        $p=~s/=list=/details/;
        $p=~s/=prop=/$member/;
        $p=~s/=action=/remote-details/;
        $p=~s/=type=/ps/;
        $p=~s/=old=//;
        $p=~s/=ask=//;
        $g.="\t-";

        #$g.="\t $z / $s / $p";        #todo

#online and status
#&print_hash(%cfg);

       # auth
       $t="$wpath/_log/group/$member.txt";
       $t=&my_read($t);
       $t=~s/\s//gs;                # no spaces or nelines
       $t=substr($t,0,5);
       $g.="\t$t ...";

       # opt status of other napp-it appliances
       #$t= @data=('ip',192.2.2.2','action','rgroup');

       #$t=&grouplib_ask_post("ip","$w[1]","action"."rgroup");
       #$g.="\t$t"; # tod0
       $g.="\t-";

       # delete member
       if ($w[1]=~/\d+\.\d+\.\d+\.\d+/) {
          $n="delete";
          $d=$url;
          $d=~s/=text=/delete/;
          $d=~s/=action=/del-nasgroupmember/;
          $d=~s/=old=/$member/;
          $d=~s/=ask=/$ask/;
          $d=~s/=ask2=/delete group-member $member ?/;
          $d=~s/=list=//;
          $d=~s/=prop=//;

          # no del link with 127.0.0.1
            if ($w[1] eq "127.0.0.1")  {$d="-"; }
       }

       # add delete option
         $g.="\t$d";

    }

#test
#&print_array(@g);


   #show as table
   $out=join("\n",@g);

    $out="<b>group-members\t<b>remote hostname</b>\t<b>details</b>\t<b>auth</b>\t<b>status</b>\t<b>action</b>\n$out";  # header
    print &list2table($out,'100%,180px,180px');

    ##
    #add new group member
    $r=$url;
         $r=~s/=text=/<b>++ add ZFS server <\/b>/;
         $r=~s/=list=//;
         $r=~s/=prop=/add-nasgroupmember/;
         $r=~s/=action=/add-nasgroupmember/;
         $r=~s/=old=//;
         print "<br>$r";

    ##


   #remote napp-it details
   #if ($in{'action'} eq "remote-details")     { &grouplib_nasgroup_details; }


   #popup/ edit-window
  # if ($in{'action'} =~/add/)                          { &grouplib_ask_property;  }

    return;

 } #/sub




#######################
 sub grouplib_nasgroup_details {            # hide: show status of remote napp-it server
#######################
    my ($group,@z,$url,$u,$n,$r,@w,@g,$w,$out,$ask);

    # show as table

    if ($in{'type'} eq "snapshot") {
         $in{'prop'}=~s/\s//g;
         $out="<b>ZFS of $in{'prop'} </b>\tUsed\tAvail\tRefer\tMounted";
         $r=&grouplib_ask_remote_snaplist($in{'prop'},,"grouplib_nasgroup_details 1916");
    }
    if ($in{'type'} eq "zfs") {
         $in{'prop'}=~s/\s//g;
         $out="<b>ZFS of $in{'prop'} </b>\tUsed\tAvail\tRefer\tMounted";
         $r=&grouplib_ask_remote_zfslist($in{'prop'},,"grouplib_nasgroup_details 1921");
    }
    if ($in{'type'} eq "ps") {
         $in{'prop'}=~s/\s//g;
         $out="<b>PID's of $in{'prop'}\tCommand (storage related only)</b>";
         $r=&grouplib_ask_remote_pslist($in{'prop'},,"grouplib_nasgroup_details 1926");
         $r=~s/;;/\n/gs;
    }

    $r=~s/,/\n/gs;
    $out.="\n$r";

    print "<br><br>\n";
    print &list2table($out,'','sort2');
}


#######################
sub grouplib_ask_property {        #hide:
#######################

    ################################
    # create form to edit a property
    ################################

    my ($ask,$close,$form,$info,$t,$opt,$r,$do,$hn);
    $close="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;member=$in{'member'}&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list=$in{'list'}\&amp;zfs=$in{'zfs'}\"><b>\&nbsp;cancel\&nbsp;/close\&nbsp;</b></a>";


    #add nas group-member
    ######################
    if ($in{'action'} eq "add-nasgroupmember") {
        $do="1";
        $form="<table><tr><td>ip: </td><td><input type='text' name='ip' value='192.168.1.' style='width: 100px'></td>";
        $form.="</tr><tr>";
        $form.="<td>member name: </td><td><input type='text' name='membername' value='' style='width: 100px'></td>";
        $form.="</tr><tr>";
        $t=&get_sha_hash(time());

        $form.="<td>auth: 256B SHA or other </td><td><input type='text' name='pw' value=\"$t\" style='width: 400px'></td>";
        $form.="</tr></table>";

        $info="Enter ip, membername and auth (must be the same as in server.auth on this server)<br>You can readd a member with same ip and different membername<br>";
    }


     ### form ###
     if ($do) {
        print &ask_pop("$form","$info","$opt");
     }

} #/sub





 ##############################
 sub grouplib_change_property {                 #hide:
 ##############################
 my ($r,$s,$t,$do,$aid,$n);

 # this part is called after user have edited a property (ex by form &ask_property above) and this change should happen now
 ##########################################################################################################################


  #del-nasgroup_member
  if ($in{'action'} eq "del-nasgroupmember") {
        $do="1";

        # delete file _log/group/$in{'old'}.txt
        $n="$wpath/_log/group/$in{'old'}.txt";
        if (-f $n) { unlink ($n); }

        $in{'action'}="";
        &reload();
  }

  #add member to nas group
  if ($in{'action'} eq "add-nasgroupmember") {

        my @files = glob("$wpath/_log/group/*.txt");

        $do="1";
        print "<br>please wait..\n";

        $in{'prop'}="'".$in{'prop'}."'";
        $in{'prop'}=~s/\'+/\'/g;
        $in{'ip'}=~s/[^0-9\.]//g;
        $in{'pw'}=~s/\s//g;

        $in{'membername'}=~s/[^a-z0-9A-Z\_\- ]/_/g;
        $in{'membername'}=~s/ +/_/g;

        if (!($in{'ip'}=~/^\d+\.\d+\.\d+\.\d+$/))   { &mess ('please enter a valid ip'); }
        if (!($in{'membername'}=~/\w+/))            { &mess ('please enter member name of remote server'); }
        if (!($in{'pw'}=~/\w+/))                    { &mess ('please enter an auth value for access to this server');   }

        # only one member per ip
        my @files = glob("$wpath/_log/group/*.txt");
        foreach my $file (@files) {
          if ($file=~/~$in{'ip'}/) { unlink($file);  }
        }

        $in{'membername'}=lc($in{'membername'});
        $file="$wpath/_log/group/$in{'membername'}"."~$in{'ip'}.txt";
        &str2file("$in{'pw'}",$file);                       # auth

        $in{'action'}="";
        &reload();
        exit;

        @post=("action","rgroup");
        $r=&grouplib_ask_post($in{'ip'},$sys{'port'});   # ip, port, par1,val1,par2,val2

        # $t="$perl -e 'print\"GET /cgi-bin/admin.pl?action=rgroup   HTTP/1.0\n\n\"' | $nc $in{'ip'} $sys{'port'}";

        if ($r=~/refused/s)         { &mess ("host $in{'ip'} is not responding on port $sys{'port'}"); }
        if (!($r=~/web-gui-ok:/s))  { &mess ("host $in{'ip'} is not a napp-it server<br>$t<hr>reply: $r"); }

        #request membership
         ###################

                # ask_remote
                # do=Larry%20Bird&age=35&pass=testing ,ip

                $in{'pw'}=~s/[^a-zA-Z0-9\,\.\-\;\:\_\+\#]//g;

                $n=`hostname`;
                chomp($n);

                $t="action=rgroup&do=request_membership\&pw=$in{'pw'}\&hostname=$n";

                # handled by -> &remote_call in admin.pl
                $r=&grouplib_ask_remote("$t","$in{'ip'}");

                # if ok: r="ok:gid" or r="wrong pw"
                if ($r=~/wrong/) { $r=~s/.*://s; &mess("$r");}

                #wenn okj dann ..ok:hostname=accessid
                $n=$r; $n=~s/^.*://s; $n=~s/=.*//;
                $aid=$r; $aid=~s/^.*=//s; $aid=~s/\s//gs;


                #write access id to file ip.txt
                $file="$wpath/_log/group/$n"."~$in{'ip'}.txt";
                &str2file("$aid",$file);

                $in{'action'}="";
                &reload();

  }  #/nasgroup

} #/sub


###########
sub grouplib_ask_post {                          #
###########
    #use LWP::UserAgent;                         #in admin.pl
    #use HTTP::Request::Common qw(POST);


    my @data=@_;                         # @data=('ip',192.2.2.2','action','rgroup');

    # ex call via  &post('ip',$ip,'key',$value,..);

    # @data=('ip,'timeout','action','rgroup','par2','val2',..)
    # bergabe array via &sub(@data);

    #use LWP::UserAgent;
    #use HTTP::Request::Common qw(POST);
    #use URI::Escape;

    # my $data = '{                                        # " in "key" or "value" is required
    #   "name": "John Doe",
    #   "age": 30
    # }';

    # URL des Webservers
    #my $url = 'https://www.example.com/api/endpoint';

    # User Agent erstellen
    #my $ua = LWP::UserAgent->new();

    # Data escaped
    # my $data_escaped = uri_escape($data);

    # Request erstellen
    #my $request = POST $url,
    #    Content => $data_parsed,               # length wird automatisch ermittelt
    #    Content_Type => 'text/plain',

    # Request senden
    #my $response = $ua->request($request);

    # Fehlerbehandlung
    #if ($response->is_error()) {
    #..

    # decode
    # use URI::Escape;

    #my $data_encoded = 'some%20text%2Bwith%20spaces%26symbols';
    #my $data_decoded = uri_unescape($data_encoded);


    shift @data;               # opt remove first two elements, otherwise use $ip=$data[1]

    my $ip =shift @data;
    my $url="http://$ip\/cgi-bin/admin.pl";

    my %data = ();  # Create an empty hash to store URL parameters

    # Populate the hash with key-value pairs (ensure `@data` is defined with key-value pairs)
    my $i = 0;
    foreach my $d (@data) {
        if ($i % 2 == 0) {
            $data{$d} = '';  # Assign empty string as initial value (optional)
        } else {
            $data{$data[$i - 1]} = $d;
        }
        $i++;
    }

    #&print_array(@data);
    #&print_hash(%data);

    $timeout=20;                 # default

   # large data, use higher timout

   my $len=length($data_escaped);
   if ($len > 64 * 1024)   { $timeout=10; }             # 64K
   if ($len > 256 * 1024)  { $timeout=15; }             # 256K
   if ($len > 1024 * 1024) { $timeout=20; }             # 1M

    # user agent
    my $ua = LWP::UserAgent->new;

    #timeout
    $ua->timeout($timeout); # post Timeout nach 120 Sekunden

    # request ex
    # my $request = POST "http://$ip\:$port/cgi-bin/admin.pl", [action=>rgroup, age => 24];

    # length is auto calculated, content-type for cgi
    # %data is a hash reference (\%data)

    my $request = POST $url,
        Content => \%data,
        Content_Type => 'application/x-www-form-urlencoded';

    my $response = $ua->request($request);                   # send


    my $headers= $response->headers_as_string;               # headers only
    my $body= $response->content;                            # body
    $body   = uri_unescape($body);

    #Connection: close
    #Date: Sun, 25 Feb 2024 14:54:05 GMT
    #Server: Apache
    #Content-Type: text/html
    #Client-Date: Sun, 25 Feb 2024 14:54:09 GMT
    #Client-Peer: 192.168.2.203:80
    #Client-Response-Num: 1
    #Client-Transfer-Encoding: chunked
    #Link: ; rel="icon"; type="image/x-icon"
    #Link: ; rel="stylesheet"; type="text/css"
    #Refresh: 3600;url=/cgi-bin/admin.pl?action=logout_admin
    #Title: omnios // ZFS appliance
    #--------------------------
    #Content-type: text/plain
    #web-gui-ok:OmniOS r151046z ..

    #<body>..</body> in $body

    #&mess("$data<hr>$headers<hr>$body");

    return ("$headers<hr>$body");

}

#########################
sub grouplib_ask_socket {        # max 32 , port 63000
#########################
  my $ip   =$_[0];
  my $data =$_[1];
  my $auth ="";                   # authvalue, add auth:=xx to data (auth in group/host~ip.txt)

  my @files = glob("$wpath/_log/group/*.txt");

  foreach my $file (@files) {
      if ($file=~/~$ip\.txt/) {
         open my $file, "<", "$file";       # file is GLOB nn warning after open ?
                $auth = <$file>;            # first line with hash
         close $file;
      }
  }
  if ($auth eq "") { return("error: auth not found"); }

  $data.=" auth:=$auth";
  # via sub socket in adminwlib

  $current{'member_ip'}=$ip;
  my $r=&exe($data);
  return ($r);

}

#EoF ######################
# do not remove line below (return true)
1;