#!/usr/bin/perl
# (c) napp-it 2012-2024 # Bookmarks: 0,0 0,0 0,0 0,0 0,0 0,0 0,0 0,0 0,0 63,4
# feb.2024
# win
# debug,    run from cli:    perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/job-scrub.pl run_id
# auto.pl: `powershell start-process  -WorkingDirectory $wpath -filepath C:\\xampp\\perl\\bin\\perl.exe -argumentlist \"$t\",\"run_$id\" -WindowStyle minimized`;       # minimized=debugmodr
#



use strict;
use Fcntl qw(:flock);                                          # filelocking
$|=1;                                                          # keine Pufferung, no buffering

use vars qw($wpath $tpath $dpath $ref $k $par $id $action $f $t @t $p @p %current $error $info %sys $member $pool);

$wpath="/xampp/web-gui";
$dpath="/xampp/web-gui/data";
$tpath="/xampp/web-gui/tmp";

chomp($wpath);      # current working dir

# common lib
require "$wpath/data/menues/_lib/windows/task-lib.pl";
require "$wpath/data/wwwroot/cgi-bin/admin-wlib.pl";       #


$par=$ARGV[0];
 if (!(defined($par))) {
     &log("job-scrub:  missing parameter");
     exit;
 }

 #par: action_job timestamp
 $par=~s/ .*//;

 # parameter: action_$job time
 if (!($par =~/\w+\_\d+$/)) {
     print "job-scrub 20: wrong par $par";
     exit;
 }
 @t=();
 @t=split(/\_/,$par);    # action job
 $action=$t[0];
 $action=~s/[^a-z]//g;

 $id=$t[1];
 #############################################


 # get par -> %current
 &task_read_jobpar($id);

# run?
if ($action eq "")                     {  print "job-scrub 42: wrong action - must be not empty\n";      exit; }
if (!("run,cancel,delete"=~/$action/)) {  print "job-scrub 43: wrong action - must be run: $action \n";  exit; }

#cancel or delete
if ($action eq "cancel") { &my_cancel; $info="cancel job"; &my_log; exit; }
if ($action =~/del/)     { &my_cancel; &my_delete; $info="delete job"; exit; }

#debug
# print "job-scrub 38: run id=$id par $par\n";

#check if pool is mounted

     $k="$id\_member";
     $member=$current{$k};
     $member=~s/.*\~//;                                   # use member ip only

     $k="$id\_pool";
     $pool=$current{$k};                                  # parameter from  &task_read_jobpar($id);

     # zfs  not mounted/zfs not available
     $t=&socket("zfs list -H -o name $pool",$member);      # cmd with/without auth,ip,timeout
     print "69 pool $t\n";
     #my $q=quotemeta($pool);

     if (!($t=~/$pool/)) {
        $info="Pool $pool not mounted";  $error="";
        &my_log("ZFS filesystem $pool not mounted/available");
        print "75 Pool=$pool not available\n$t\n";
        exit;
     }

# fork parent=monitor, child=task;  #my parent=$$;
my $child=fork();

if ($child) {    #
    &my_run;
    wait;
} else {
    &my_monitor;
}
# print "job-scrub: end\n";
# &my_log;

 exit;


 ## run job (child process)
 sub my_run {
 ############

     my $r="$id\_member";
     $member=$current{$r};
     $member=~s/.*~//;          # use ip

     print "$member: zpool scrub $pool\n";
     $error=&socket("zpool scrub $pool",$member);
     if ($error=~/cannot open/s) { $info=$error; &my_log; exit; }

 }


################# monitor job (parent process)
 sub my_monitor {
#################
my ($r,$p,$pool);

     $r="$id\_pool";
     $pool=$current{$r};

    #debug
    #print "job-scrub: monitor\n";
    sleep(6);                                                # wait for run to be started

    while (1) {

        $r=&socket("zpool status $pool",$member);
        if ($r=~/cannot open/s) { $error="error"; $info=$r; &my_log; exit; }

        #debug
        print "122 par ent $r\n";

        $r=~s/.*scan: //s;
        $r=~s/config:.*//s;

        #log info
        $info="$r";

        #progress in file /tmp/nappit/id.progress
        if ($r=~/\% done/s) {
           $p=$r;
           chomp($p);
           $p=~s/\%.*//s;
           $p=~s/.* //s;        # keep percent value
           #$p=~s/\n/ _n /gs;
           &my_write("$wpath/tmp/$id.progress",$p);
        } else {
           &my_log;
           exit;      # finished
        }
        &my_wait_and_check(10);   # check cancel, wait 10s
    }
 }


## log last run in jobfile /_log/jobs/id...job
sub my_log {
###########
  # line 1: lastrun = ok or err
  # line 2: time of last run
  # line 3+: info
  &task_gettime;                                             # $current{'timestamp'}, $current{'logtime'}

  my $log="$wpath/_log/jobs/$id.log";
  my (@v,$v,$n,$l,$f1,$f2,@data);

  $l=$error;
  if ($l eq "") { $l="ok"; }
  $l=~s/\n/ /gs;

  #update job.par  $current{'logtime'};
  $t="$id"."_last";                                        # update last
  $current{$t}=$current{'logtime'};
  &task_update_par($id);

  #rm progress
  if (-f "$wpath/tmp/$id.progress") {  unlink("$wpath/tmp/$id.progress"); }

  $info=~s/\n.*//s;                                        # first line

  #log newest at top
  ###################
  open (F,"<$log");
        flock(F, LOCK_EX);
        @data=<F>;
  close (F);

  #unshift   $l=ok/error
  unshift(@data,"$l; time: $current{'timestamp'}; info: $member $info\n");

  if (@data > 50) {
             $t=@data-50;
             splice(@data,50,$t);
  }

 $t=join("",@data);
 open (PF, ">$log");
    flock(PF, LOCK_EX);
    print PF "$t";
 close (PF);

 #stage 1: log general jobs
 ###########################
  my $log="$wpath/_log/jobs/jobs.log";
  my $lock="$wpath/_log/jobs/jobs.lock";
  open (X,"<$lock");
  flock(X, LOCK_EX);
          open (F,"<$log");
              flock(F, LOCK_EX);
              @data=<F>;
          close (F);

        #unshift   $l=ok/error
        unshift(@data,"$l; scrub $ref $id; time: $current{'timestamp'}; info: $member $info\n");

        if (@data > 50) {
                   $t=@data-50;
                   splice(@data,50,$t);
        }

       $t=join("",@data);
       open (PF, ">$log");
          flock(PF, LOCK_EX);
          print PF "$t";
       close (PF);
 close (X);

}

######################
sub my_wait_and_check {  # monitor wait and rech cancel/del
#######################
    my $t=$_[0];
    while ($t>0) {
       if (-f "$wpath/tmp/$id.cancel")    { &my_cancel; exit; }
       if (-f "$wpath/tmp/$id.delete")    { &my_cancel; &my_delete; exit; }
       sleep (2);
       $t=$t-2;
    }
}


## cancel job
sub my_cancel  {
###############
   my $k="$id\_pool";
   my $pool=$current{$k};
   my $error=&socket("zpool scrub -s $pool",$member);
   if (-f "$tpath/tmp/$id.progress")  {  unlink("$tpath/$id.progress"); }
   if (-f "$tpath/tmp/$id.cancel")    {  unlink("$tpath/$id.cancel"); }
 }

 ## delete job
 sub my_delete {
 ##############
 my ($r);
      if ($id ne "") {
      $r=&socket("Remove-Item -Path \"$wpath/_log/jobs/$id.*\" -Force","127.0.0.1");          # jobfile is local, use powershell

        print "##### job-scrub 220: delete $id ######  $r\n";
      } else {
        print "job-scrub delete, no id !! ########### \n";
      }
 }